/**
 * Using CT-API
 *
 * Introduction Introduction
 *
 * This documentation covers the APIs provided by ACS CT-API library. This
 * library implements CT-API functions for ACS smart card readers.
 *
 * DefiningCardTerminals Defining Card Terminals
 *
 * To use CT-API with ACS smart card readers, you must place a initialization
 * file "ctacs.ini" to the current directory with your application program. This
 * file is to map PC/SC readers to card terminal numbers and ICC interfaces.
 *
 * The following sample INI file defines 2 card terminals. CTN1 is mapped to
 * ACR38U-CCID while CTN2 is mapped to ACR128U and each ICC interface is mapped
 * to PC/SC reader name.
 *
 * LinuxMacOSX Linux/Mac OS X
 *
 * ; Sample ctacs.ini (Linux/Mac OS X)
 *
 * [CardTerminal]
 * CTN1=ACR38U-CCID
 * CTN2=ACR128U
 *
 * [ACR38U-CCID]
 * ICC1=ACS ACR38U-CCID 00 00
 *
 * [ACR128U]
 * ICC1=ACS ACR128U 00 00
 * ICC2=ACS ACR128U 00 01
 * ICC3=ACS ACR128U 00 02
 *
 * CallingCTAPIFunctions Calling CT-API Functions
 *
 * Your source code must include a header file "ct_api.h" in order to call the
 * CT-API functions.
 *
 */
 
#include <stdio.h>
#include <ct_api.h>

int main(int argc, char *argv[])
{
    int i;
    char ret;
    unsigned short ctn;
    unsigned short pn;
    unsigned char sad;
    unsigned char dad;

    // REQUEST ICC
    unsigned char command[] = { 0x20, 0x12, 0x01, 0x00, 0x00 };
    unsigned short lenc = sizeof(command);

    unsigned char response[300];
    unsigned short lenr = sizeof(response);

    ctn = 1;
    pn = 1;

    // Initialize card terminal
    ret = CT_init(ctn, pn);
    if (ret != OK)
    {
        printf("Error: CT_init failed with error %d\n", ret);
        return 1;
    }

    sad = 2; // Source = Host
    dad = 1; // Destination = Card Terminal

    // Send command
    ret = CT_data(ctn, &dad, &sad, lenc, command, &lenr, response);
    if (ret != OK)
        printf("Error: CT_data failed with error %d\n", ret);
    else
    {
        // Display response
        printf("Response: ");
        for (i = 0; i < lenr; i++)
            printf("%02X ", response[i]);
        printf("\n");
    }

    // Close card terminal
    ret = CT_close(ctn);
    if (ret != OK)
        printf("Error: CT_close failed with error %d\n", ret);

    return 0;
}

/**
 * @page ReturnValues Return Values
 *
 * @section CTAPIReturnValues CT-API Return Values
 *
 * CT-API functions return the following values:
 *
 * <table>
 * <tr><th>Error Code</th><th>Value</th><th>Description</th></tr>
 * <tr><td>OK</td><td>0</td><td>Function call was successful.</td></tr>
 * <tr><td>ERR_INVALID</td><td>-1</td><td>Invalid parameter or value.</td></tr>
 * <tr><td>ERR_CT</td><td>-8</td><td>CT error.</td></tr>
 * <tr><td>ERR_TRANS</td><td>-10</td><td>Non-eliminable transmission error.</td></tr>
 * <tr><td>ERR_MEMORY</td><td>-11</td><td>Memory assignment error in HTSI.</td></tr>
 * <tr><td>ERR_HOST</td><td>-127</td><td>Abort of function by host/OS.</td></tr>
 * <tr><td>ERR_HTSI</td><td>-128</td><td>HTSI error.</td></tr>
 * </table>
 */
