#ifndef CARTEMIFARE_H
#define CARTEMIFARE_H

#include <nfc/nfc.h>

#include <QStringList>
#include <QString>
#include <QByteArray>

#include <stdint.h>

// Compiler directive, set struct alignment to 1 uint8_t for compatibility
#pragma pack(1)

typedef enum                // page 14 Mifare-MF1S50YYX.pdf
{
    MC_SPECIAL_WRITE = 0x00,
    MC_DATA       = 0x01,
    MC_REQA       = 0x26,   // 7 bits
    MC_READ       = 0x30,
    MC_UNLOCK1    = 0x40,   // Personalize UID usage
    MC_WIPE       = 0x41,
    MC_UNLOCK2    = 0x43,
    MC_HALT       = 0x50,   // 0x50 0x00 HLTA (ISO14443-3)
    MC_WAKE_UP    = 0x52,   // WUPA (ISO14443)
    MC_AUTH_A     = 0x60,   // Authentication command (API-ACR122U-2.02.pdf : page 13)
    MC_AUTH_B     = 0x61,    
    MC_SELECT_ALL = 0x93,   // 0x93 0x70 Select CL1
    MC_CL1_SELECT = 0x94,   // 0x93 0x20 Anticollision CL1
    MC_CL2        = 0x95,   // Anticollision CL2
    MC_CL3_AC     = 0x97,
    MC_WRITE      = 0xA0,
    MC_TRANSFER   = 0xB0,   // Read Binary Blocks (API-ACR122U-2.02.pdf : page 16)
    MC_DECREMENT  = 0xC0,
    MC_INCREMENT  = 0xC1,
    MC_STORE      = 0xC2,   // S-block DESELECT
    MC_RATS       = 0xE0,   // RATS (ISO14443-4)
    MC_BLANK      = 0xFF
} MifareCommande; // mifare_cmd;

// MIFARE command params
typedef struct
{
    uint8_t  key[6];
    uint8_t  authUid[4];
} MifareParametresAuth; // mifare_param_auth

typedef struct
{
  uint8_t  data[16];
} MifareParametresData; // mifare_param_data

typedef struct
{
  uint8_t  value[4];
} MifareParametresValue; // mifare_param_value

typedef union
{
  MifareParametresAuth mpAuthentication;
  MifareParametresData mpData;
  MifareParametresValue mpValue;
} MifareParametres; //mifare_param

// MIFARE Classic
typedef struct
{
  uint8_t  uid[4];
  uint8_t  bcc;
  uint8_t  inconnu1;
  uint8_t  atqa[2];
  uint8_t  inconnu2[8];
} MifareBlocFabricant; //mifare_classic_block_manufacturer;

typedef struct
{
  uint8_t  data[16];
} MifareBlocData; //mifare_classic_block_data;

typedef struct
{
  uint8_t  keyA[6];
  uint8_t  accessBits[4];
  uint8_t  keyB[6];
} MifareBlocSecurite; //mifare_classic_block_trailer;

typedef union
{
  MifareBlocFabricant mbFabricant;
  MifareBlocData mbData;
  MifareBlocSecurite mbSecurite;
} MifareBloc; //mifare_classic_block;

/*typedef struct
{
  MifareBloc amb[256];
} MifareCarte; //mifare_classic_tag;*/

// Reset struct alignment to default
#  pragma pack()

// Tableau de clés pour Mifare Classic
static const uint8_t defaultKeys[][6] =
{
    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff}, // Clé par défaut
    {0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5}, // Clé NFCForum MAD
    {0xd3, 0xf7, 0xd3, 0xf7, 0xd3, 0xf7}, // Clé NFCForum content
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // Clé vide
    {0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5},
    {0x4d, 0x3a, 0x99, 0xc3, 0x51, 0xdd},
    {0x1a, 0x98, 0x2c, 0x7e, 0x45, 0x9a},
    {0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff},
    {0x71, 0x4c, 0x5c, 0x88, 0x6e, 0x97},
    {0x58, 0x7e, 0xe5, 0xf9, 0x35, 0x0f},
    {0xa0, 0x47, 0x8c, 0xc3, 0x90, 0x91},
    {0x53, 0x3c, 0xb6, 0xc7, 0x23, 0xf6},
    {0x8f, 0xd0, 0xa4, 0xf2, 0x56, 0xe9},
    {0x6a, 0x19, 0x87, 0xc4, 0x0a, 0x21}, // Clé Badge (A)
    {0x7f, 0x33, 0x62, 0x5b, 0xc1, 0x29}  // Clé Badge (B)
};

typedef enum
{
    CLE_DEFAUT, // Clé par défaut
    CLE_NFCFORUM_MAD,
    CLE_NFC_FORUM_CONTENT,
    CLE_VIDE,
    CLE_DEFAUT_1,
    CLE_DEFAUT_2,
    CLE_DEFAUT_3,
    CLE_DEFAUT_4,
    CLE_DEFAUT_5,
    CLE_DEFAUT_6,
    CLE_DEFAUT_7,
    CLE_DEFAUT_8,
    CLE_DEFAUT_9,
    CLE_BADGE_A,
    CLE_BADGE_B
} MifareNumCle;

typedef struct
{
    uint8_t  key[6];
} MifareCle;

#define NB_MIFARE_CLES    15

// Mifare Classic 1k
#define NB_BLOCS    64
#define NB_CLES     (NB_BLOCS/4)

class CarteMifare
{
public:
    CarteMifare();
    CarteMifare(MifareNumCle numCle);
    CarteMifare(char typeAccesCle, MifareNumCle numCle);
    ~CarteMifare();

    void             effacerBlocsLus();
    void             effacerBlocLu(int numeroBloc);
    MifareParametres getParametres() const;
    void             setParametres(MifareParametres parametres);
    void             afficherParametresAuthentication() const;
    nfc_modulation   getNfcModulation() const { return nm; }
    char             getTypeAccesCle() const { return typeAccesCle; }
    void             setTypeAccesCle(char typeAccesCle);
    MifareCle        getCle(char typeAccesCle='A', int numeroBloc=0) const;
    void             setCle(MifareNumCle numCle, int numeroBloc=0);
    void             setCle(uint8_t cle[], int numeroBloc=0);
    void             setCle(char typeAccesCle, MifareNumCle numCle, int numeroBloc=0);
    void             setCle(char typeAccesCle, uint8_t cle[], int numeroBloc=0);
    int              getNbCles() const { return nbCles; }
    int              getNbBlocs() const { return nbBlocs; }
    int              getNbBlocsLus() const { return nbBlocsLus; }
    int              getNbSecteurs() const { return nbBlocs/4; }
    MifareBloc       getBloc(int numeroBloc) const;
    MifareBloc       getBlocSecurite(int numeroBloc) const;
    MifareBloc       getBlocFabricant() const;
    void             setBloc(int numeroBloc, MifareBloc bloc);
    void             afficherBlocs() const;
    QString          getBlocToString(int numeroBloc) const;
    QString          getBlocToHexaString(int numeroBloc) const;
    QString          afficherBloc(int numeroBloc) const;
    void             afficherBlocSecurite(int numeroBloc) const;
    void             afficherBlocFabricant() const;
    bool             estPremierBloc(int numeroBloc) const;
    bool             estBlocSecurite(int numeroBloc) const;
    int              getNumeroBlocSecurite(int numeroBloc) const;
    static void      printHexa(const uint8_t *pbtData, const size_t szBytes, QString label = "");
    static QString   toQStringHexa(const uint8_t *pbtData, const size_t szBytes, QString label = "", QString delimiteur = "");
    static QString   toQStringNum(const uint8_t *pbtData, const size_t szBytes, QString label = "");
    static void      toBytes(uint32_t data, uint8_t *pbtData, size_t *szBytes);
    static int       toInt(const uint8_t *pbtData, const size_t szBytes);
    void             decoderACL(const uint8_t acl[]) const;
    /*void             setUid(uint32_t uid);
    void             setUid(QString uid);
    void             setUidFromHexa(QString uid);*/
    uint32_t         getUid();
    QString          getUidToString();
    QString          getUidToHexa();

private:
    MifareBloc       blocs[NB_BLOCS]; // 1k = 64 x 16
    MifareParametres parametres;
    nfc_modulation   nm;
    char             typeAccesCle;
    bool             blocsLus[NB_BLOCS]; // 1k = 64 x 16
    int              nbBlocsLus;
    const int        nbBlocs;
    const int        nbCles;
    uint8_t          clesA[NB_CLES][6]; // une clé par secteur 1k = (64 x 16)/4
    uint8_t          clesB[NB_CLES][6]; // une clé par secteur 1k = (64 x 16)/4
    uint32_t         uid;

    QString          bin2str(bool C1, bool C2, bool C3) const;
    uint8_t          bin2byte(bool C1, bool C2, bool C3) const;
};

#endif // CARTEMIFARE_H
